@@ -22,8 +22,27 @@
 #include <stdlib.h>
 #include <assert.h>
 
-#ifdef __SSE4_1__
+/* Check for SSE4.1 support */
+#if defined(__SSE4_1__)
 #include <smmintrin.h>
+#define USE_SSE4_1 1
+#else
+#define USE_SSE4_1 0
+#endif
+
+/* Check for other SIMD instruction sets */
+#if defined(__AVX__) && defined(__AVX2__)
+#include <immintrin.h>
+#define USE_AVX2 1
+#else
+#define USE_AVX2 0
+#endif
+
+#if defined(__ARM_NEON__) || defined(__ARM_NEON)
+#include <arm_neon.h>
+#define USE_NEON 1
+#else
+#define USE_NEON 0
 #endif
 
 #include <frei0r.h>
@@ -46,17 +65,17 @@ int f0r_init()
 void f0r_deinit()
 { /* no initialization required */ }
 
-void f0r_get_plugin_info(f0r_plugin_info_t* tint0r_instance_t)
+void f0r_get_plugin_info(f0r_plugin_info_t* info)
 {
-  tint0r_instance_t->name = "Tint0r";
-  tint0r_instance_t->author = "Maksim Golovkin & Cynthia";
-  tint0r_instance_t->plugin_type = F0R_PLUGIN_TYPE_FILTER;
-  tint0r_instance_t->color_model = F0R_COLOR_MODEL_BGRA8888;
-  tint0r_instance_t->frei0r_version = FREI0R_MAJOR_VERSION;
-  tint0r_instance_t->major_version = 0;
-  tint0r_instance_t->minor_version = 1;
-  tint0r_instance_t->num_params = 3;
-  tint0r_instance_t->explanation = "Tint a source image with specified colors";
+  info->name = "Tint0r";
+  info->author = "Maksim Golovkin & Cynthia";
+  info->plugin_type = F0R_PLUGIN_TYPE_FILTER;
+  info->color_model = F0R_COLOR_MODEL_BGRA8888;
+  info->frei0r_version = FREI0R_MAJOR_VERSION;
+  info->major_version = 0;
+  info->minor_version = 1;
+  info->num_params = 3;
+  info->explanation = "Tint a source image with specified colors";
 }
 
 void f0r_get_param_info(f0r_param_info_t* info, int param_index)
@@ -143,70 +162,44 @@ void f0r_get_param_value(f0r_instance_t instance,
   }
 }
 
-#ifndef __SSE4_1__
 static inline unsigned char map_color(double amount, double comp_amount, float color, float luma, float minColor, float maxColor)
 {
   double val = (comp_amount * color) + amount * (luma * (maxColor - minColor) + minColor);
   return (unsigned char)(255*CLAMP(val, 0, 1));
 }
-#endif
 
-void f0r_update(f0r_instance_t instance, double time,
-                const uint32_t* inframe, uint32_t* outframe)
+#if USE_SSE4_1
+static void tint_sse41(const uint32_t* inframe, uint32_t* outframe, size_t len,
+                       double amount, f0r_param_color_t blackColor, f0r_param_color_t whiteColor)
 {
-  assert(instance);
-  tint0r_instance_t* inst = (tint0r_instance_t*)instance;
-
-  #ifdef __SSE4_1__
-  size_t len = (inst->width * inst->height) / 4;
-
-  const __m128 weights = _mm_set_ps(0.0, 0.299, 0.587, 0.114),
-  amount = _mm_set1_ps(inst->amount),
+  const __m128 weights = _mm_set_ps(0.0, 0.114, 0.587, 0.299),
+  sse_amount = _mm_set1_ps(amount),
   /* Pass the alpha channel */
   comp_amount = _mm_set_ps(1.0,
-                           1.0 - inst->amount,
-                           1.0 - inst->amount,
-                           1.0 - inst->amount);
-
-  f0r_param_color_t black = inst->blackColor,
-  white  = inst->whiteColor;
+                           1.0 - amount,
+                           1.0 - amount,
+                           1.0 - amount);
 
   /* Zero the alpha component to exclude it from calculations. */
-  const __m128 cmin = _mm_set_ps(0.0, black.r, black.g, black.b),
-  cdelta = _mm_sub_ps(_mm_set_ps(0.0, white.r, white.g, white.b), cmin),
-  tmp0 = _mm_mul_ps(cdelta, amount),
-  tmp1 = _mm_mul_ps(_mm_mul_ps(amount, _mm_set1_ps(255.0)), cmin);
+  const __m128 cmin = _mm_set_ps(0.0, blackColor.b, blackColor.g, blackColor.r),
+  cdelta = _mm_sub_ps(_mm_set_ps(0.0, whiteColor.b, whiteColor.g, whiteColor.r), cmin),
+  tmp0 = _mm_mul_ps(cdelta, sse_amount),
+  tmp1 = _mm_mul_ps(_mm_mul_ps(sse_amount, _mm_set1_ps(255.0)), cmin);
 
   __m128 p, p0, p1, p2, p3, luma;
-  #else
-  unsigned int len = inst->width * inst->height;
-  double amount = inst->amount;
-  double comp_amount = 1.0 - inst->amount;
 
-  unsigned char* dst = (unsigned char*)outframe;
-  const unsigned char* src = (unsigned char*)inframe;
-  float b, g, r;
-  float luma;
-  #endif
-
-  while (len--)
+  // Process pixels in groups of 4
+  for (size_t i = 0; i < len; i++)
   {
-    #ifdef __SSE4_1__
     /* Load four pixels at once. */
-    p = _mm_loadu_si128((__m128i*)inframe);
+    p = _mm_loadu_si128((__m128i*)(inframe + i * 4));
 
     /* Extract four pixels into separate XMM registers and convert them to float. */
     p0 = _mm_cvtepi32_ps(_mm_cvtepu8_epi32(p));
     p1 = _mm_cvtepi32_ps(_mm_cvtepu8_epi32(_mm_srli_si128(p, 4)));
     p2 = _mm_cvtepi32_ps(_mm_cvtepu8_epi32(_mm_srli_si128(p, 8)));
     p3 = _mm_cvtepi32_ps(_mm_cvtepu8_epi32(_mm_srli_si128(p, 12)));
-    #else
-    b = *src++ / 255.;
-    g = *src++ / 255.;
-    r = *src++ / 255.;
-    #endif
 
-    #ifdef __SSE4_1__
     #define tint(v) \
       luma = _mm_dp_ps((v), weights, 0x7F); \
       v = _mm_add_ps(_mm_mul_ps(comp_amount, (v)), \
@@ -219,19 +212,81 @@ void f0r_update(f0r_instance_t instance, double time,
     p = _mm_packus_epi16(_mm_packus_epi32(p0, p1),
                          _mm_packus_epi32(p2, p3));
 
-    _mm_storeu_si128((__m128i*)outframe, p);
+    _mm_storeu_si128((__m128i*)(outframe + i * 4), p);
+  }
+}
+#elif USE_AVX2
+static void tint_avx2(const uint32_t* inframe, uint32_t* outframe, size_t len,
+                      double amount, f0r_param_color_t blackColor, f0r_param_color_t whiteColor)
+{
+  // AVX2 implementation would go here
+  // For now, fall back to scalar implementation
+  // This is a placeholder for a future AVX2 implementation
+}
+#elif USE_NEON
+static void tint_neon(const uint32_t* inframe, uint32_t* outframe, size_t len,
+                      double amount, f0r_param_color_t blackColor, f0r_param_color_t whiteColor)
+{
+  // NEON implementation would go here
+  // For now, fall back to scalar implementation
+  // This is a placeholder for a future NEON implementation
+}
+#endif
 
-    /* Stride of 128 bits; i.e. 16 bytes */
-    inframe += 4;
-    outframe += 4;
-    #else
-    luma = (b * .114 + g * .587 + r * .299);
+static void tint_scalar(const uint32_t* inframe, uint32_t* outframe, size_t len,
+                        double amount, f0r_param_color_t blackColor, f0r_param_color_t whiteColor)
+{
+  double comp_amount = 1.0 - amount;
 
-    *dst++ = map_color(amount, comp_amount, b, luma, inst->blackColor.b, inst->whiteColor.b);
-    *dst++ = map_color(amount, comp_amount, g, luma, inst->blackColor.g, inst->whiteColor.g);
-    *dst++ = map_color(amount, comp_amount, r, luma, inst->blackColor.r, inst->whiteColor.r);
-    *dst++ = *src++;
-    #endif
+  const unsigned char* src = (const unsigned char*)inframe;
+  unsigned char* dst = (unsigned char*)outframe;
+
+  float b, g, r;
+  float luma;
+
+  while (len--)
+  {
+    b = src[0] / 255.0f;
+    g = src[1] / 255.0f;
+    r = src[2] / 255.0f;
+
+    luma = (b * 0.114f + g * 0.587f + r * 0.299f);
+
+    dst[0] = map_color(amount, comp_amount, b, luma, blackColor.b, whiteColor.b);
+    dst[1] = map_color(amount, comp_amount, g, luma, blackColor.g, whiteColor.g);
+    dst[2] = map_color(amount, comp_amount, r, luma, blackColor.r, whiteColor.r);
+    dst[3] = src[3]; // Copy alpha
+
+    src += 4;
+    dst += 4;
   }
 }
 
+void f0r_update(f0r_instance_t instance, double time,
+                const uint32_t* inframe, uint32_t* outframe)
+{
+  assert(instance);
+  tint0r_instance_t* inst = (tint0r_instance_t*)instance;
+  size_t len = inst->width * inst->height;
+
+#if USE_SSE4_1
+  // Process in chunks of 4 pixels for SSE
+  size_t sse_len = len / 4;
+  size_t remainder = len % 4;
+
+  if (sse_len > 0) {
+    tint_sse41(inframe, outframe, sse_len, inst->amount, inst->blackColor, inst->whiteColor);
+  }
+
+  // Handle remaining pixels with scalar implementation
+  if (remainder > 0) {
+    const uint32_t* remaining_in = inframe + (sse_len * 4);
+    uint32_t* remaining_out = outframe + (sse_len * 4);
+    tint_scalar(remaining_in, remaining_out, remainder, inst->amount, inst->blackColor, inst->whiteColor);
+  }
+#else
+  // Use scalar implementation for all pixels
+  tint_scalar(inframe, outframe, len, inst->amount, inst->blackColor, inst->whiteColor);
+#endif
+}
+
